#!/usr/bin/env python3

"""Common class for working with Carbon Black Envs"""

## stardard library
from dataclasses import dataclass
from enum import Enum
import logging
import os
## third party modules
from botocore.exceptions import ProfileNotFound
import boto3
## globals
logger = logging.getLogger(__name__)
logger.setLevel(level=logging.DEBUG)


class Platform(str, Enum):
    gcp = "gcp"
    aws = "aws"


@dataclass
class Account:
    accountAlias: str

    @property
    def awsRegions(cls):
        match cls.accountAlias:
            case "cb-dev":
                return ['us-west-2', 'us-east-2']
            case "confer-test":
                return ['us-west-2', 'us-east-1']
            case "scargo":
                return ['us-east-1','ap-southeast-2','ap-northeast-1','eu-central-1','eu-west-2','us-west-2']
            case _:
                return ['us-east-1', 'us-west-2']

    @property
    def awsProfile(cls):
        match cls.accountAlias:
            case "cb-dev":
                return os.environ.get("CB_DEV_PROFILE", "default")
            case "confer-test":
                return os.environ.get("CONFER_TEST_PROFILE", "default")
            case "scargo":
                return os.environ.get("SCARGO_PROFILE", "default")
            case _:
                return os.environ.get("AWS_DEFAULT_PROFILE", "default")

    @property
    def session(cls) -> boto3.session.Session:
        args={}
        if cls.awsProfile: args["profile_name"] = cls.awsProfile
        if cls.awsRegions[0]:  args["region_name"] = cls.awsRegions[0]
        logger.debug("AWS Session Args: %s" % args)
        try:
            return boto3.session.Session(**(args))
        except ProfileNotFound:
            args.pop("profile_name", None)
            logger.debug("New AWS Default Session Args: %s" % args)
            return boto3.session.Session(**(args))


    def setDefault(cls, awsRegion=None) -> None:
        """
        sets AWS Boto3 default session
        """
        args={}
        if cls.awsProfile: args["profile_name"] = cls.awsProfile
        if awsRegion:  args["region_name"] = awsRegion
        logger.debug("AWS Session Args: %s" % args)
        try:
            boto3.setup_default_session(**(args))
        except ProfileNotFound:
            args.pop("profile_name", None)
            logger.debug("New AWS Default Session Args: %s" % args)
            boto3.setup_default_session(**(args))


@dataclass
class CbcEnvironment:
    '''
    Returns a CbcEnvironment class for tracking CBC OrgId Environment details.
    '''
    _name: str

    def __str__(cls):
        return f'{type(cls).__name__}({cls.name})'

    def __post_init__(cls):
        cls._name = cls._name.lower()

    @property
    def name(cls):
        try:
            return {
                "kcdev01":   "dev01",
                "kctest03":  "vfyus",
                "eap1use1":  "eap01",
                "prd1use1":  "prod01",
                "prd2use1":  "prod02",
                "prd5use1":  "prod05",
                "prd1ec1":   "prod06",
                "prd1apne1": "prodnrt",
                "prd1apse2": "prodsyd",
                "prd1apse2": "prodsyd",
            }[cls._name]
        except KeyError:
            return cls._name

    @property
    def kcenv(cls):
        try:
            return {
                "dev01": "kcdev01",
                "test03": "kctest03",
                "eap01": "eap1use1",
                "prod01": "prd1use1",
                "prod02": "prd2use1",
                "prod05": "prd5use1",
                "prod06": "prd1ec1",
                "prodnrt": "prd1apne1",
                "prodsyd": "prd1apse2",
                "prodsyd": "prd1apse2",
            }[cls.name]
        except KeyError:
            return cls._name

    @property
    def stage(cls):
        match cls.name:
            case ('test03'):
                return 'test'
            case ('int1usw2'):
                return 'test'
            case ('int2use2'):
                return 'int'
            case ('scl2use2'):
                return 'scl'
            case ('eap01'|'prod01'|'prod02'|'prod05'|'prod06'|'prodnrt'|'prodsyd'|'prd1ew2'):
                return 'prod'
            case _:
                return 'dev'

    @property
    def region(cls):
        match cls.name:
            case ('int1usw2'):
                return 'us-west-2'
            case ('int2use2'|'scl2use2'):
                return 'us-east-2'
            case ('prod06'):
                return 'eu-central-1'
            case ('prodnrt'):
                return 'ap-northeast-1'
            case ('prodsyd'):
                return 'ap-southeast-2'
            case ('prd1ew2'):
                return 'eu-west-2'
            case _:
                return 'us-east-1'

    @property
    def awsAccountId(cls):
        match cls.name:
            case ('scl2use2'|'int2use2'):
                return '722037928156'
            case ('eap01'|'prod01'|'prod02'|'prod05'|'prod06'|'prodnrt'|'prodsyd'|'prd1ew2'):
                return '132308400445'
            case _:
                return '605728677638'

    @property
    def awsAccountAlias(cls):
        try:
            return {
                '722037928156': 'cb-dev',
                '132308400445': 'scargo',
                '605728677638': 'confer-test',
            }[cls.awsAccountId]
        except KeyError:
            return cls.awsAccountId


def getCbcEnvs(env_names):
    '''
    Returns a list of CbcEnvironment objects for each environment name string passed.
    '''
    envs = []
    for env in env_names:
        try:
            envs.append(CbcEnvironment(env))
        except KeyError:
            raise Exception("Environment not in list %s" % [e for e in map] )
    return envs
