#!/usr/bin/env python3

"""Common class for working with Carbon Black Envs"""

## stardard library
from dataclasses import dataclass
from typing import List, Dict
import csv
import logging

## third party modules
from rich.console import Console
from rich.table import Table
import boto3
import botocore

## custom modules
from cb_dba.CarbonBlack import CbcEnvironment, Account

logger = logging.getLogger(__name__)
logger.setLevel(level=logging.DEBUG)
console = Console()


def dictToCsv(inventoryDicts: List[Dict], filename):
    headers = inventoryDicts[0].keys()
    with open(filename, 'w', newline='') as output_file:
        dict_writer = csv.DictWriter(output_file, headers)
        dict_writer.writeheader()
        dict_writer.writerows(inventoryDicts)


def printInventory(title: str, inventoryDicts: Dict):
    headers = inventoryDicts[0].keys()
    table = Table(title=title, *headers)
    for item in inventoryDicts:
        # all cells have to contain strings
        table.add_row(*[str(v) for v in item.values()])
    console.print(table)


@dataclass
class AwsOpensearchDomain:
    client: 'boto3.client.OpenSearchService'
    AccountId: str
    AccountName: str
    Region: str
    DomainName: str
    EngineType: str

    def __post_init__(cls):
        cls.describe()

    def describe(cls):
        response = cls.client.describe_domain(
            DomainName=cls.DomainName
        )
        info = response['DomainStatus']
        object.__setattr__(cls, 'info', info)
        tags = cls.client.list_tags(ARN=cls.info['ARN'])['TagList']
        object.__setattr__(cls, 'tags', tags)

    @property
    def EngineVersion(cls):
        return cls.info['EngineVersion']

    @property
    def InstanceType(cls):
        return cls.info['ClusterConfig']['InstanceType']

    @property
    def InstanceCount(cls):
        return cls.info['ClusterConfig']['InstanceCount']

    @property
    def DedicatedMasterEnabled(cls):
        return cls.info['ClusterConfig']['DedicatedMasterEnabled']

    @property
    def DedicatedMasterType(cls):
        if cls.DedicatedMasterEnabled:
            return cls.info['ClusterConfig']['DedicatedMasterType']
        else:
            return "n/a"

    @property
    def DedicatedMasterCount(cls):
        if cls.DedicatedMasterEnabled:
            return cls.info['ClusterConfig']['DedicatedMasterCount']
        else:
            return "n/a"

    @property
    def Ownername(cls):
        try:
            return next(item for item in cls.tags if item["Key"] == "owner")['Value']
        except Exception:
            return "missing owner"

    def output(cls) -> str:
        return {
            'DomainName': cls.DomainName,
            'AccountId': cls.AccountId,
            'AccountName': cls.AccountName,
            'Region': cls.Region,
            'EngineVersion': cls.EngineVersion,
            'InstanceType': cls.InstanceType,
            'InstanceCount': cls.InstanceCount,
            'DedicatedMasterEnabled': cls.DedicatedMasterEnabled,
            'DedicatedMasterType': cls.DedicatedMasterType,
            'DedicatedMasterCount': cls.DedicatedMasterType,
            'Ownername': cls.Ownername,
        }

    def writeCsv(cls, filename):
        dictToCsv(cls.output(), filename)


@dataclass
class CbcElasticsearch:
    environment: CbcEnvironment
    esId: str
    esServerType: str

    def __post_init__(cls):
        for k,v in cls.regionDetails.items():
            object.__setattr__(cls, k, v)

    @property
    def regionDetails(cls):
        return {
            'AccountId': cls.environment.awsAccountId,
            'AccountName': cls.environment.awsAccountAlias,
            'Region': cls.environment.awsRegion
        }

    @property
    def Name(cls):
        return cls.environment.name.lower()

    @property
    def Port(cls):
        return '9300'

    @property
    def Port(cls):
        return '9300'

    @property
    def Protocol(cls):
        return 'http'

    @property
    def EndpointUrls(cls, nodes=3):
        return [f'{cls.Protocol}://coordinator{node}-{cls.esServerType.lower()}-{cls.Name}.{cls.esId.lower()}.cbd.internal:{cls.Port}' for node in range(0,nodes)]

    @property
    def EngineVersion(cls):
        return cls.esId

    @property
    def InstanceType(cls):
        return "n/a"

    @property
    def InstanceCount(cls):
        return "n/a"

    @property
    def DedicatedMasterEnabled(cls):
        return "n/a"

    @property
    def DedicatedMasterType(cls):
        return "n/a"

    @property
    def DedicatedMasterCount(cls):
        return "n/a"

    @property
    def Ownername(cls):
        return "Data Engineering / Defense Guardians"

    def output(cls) -> str:
        return {
            'Name': cls.Name,
            'AccountId': cls.AccountId,
            'AccountName': cls.AccountName,
            'Region': cls.Region,
            'EndpointUrls': cls.EndpointUrls,
            'EngineVersion': cls.EngineVersion,
            'InstanceType': cls.InstanceType,
            'InstanceCount': cls.InstanceCount,
            'DedicatedMasterEnabled': cls.DedicatedMasterEnabled,
            'DedicatedMasterType': cls.DedicatedMasterType,
            'DedicatedMasterCount': cls.DedicatedMasterType,
            'Ownername': cls.Ownername,
        }

    def writeCsv(cls, filename):
        dictToCsv(cls.output(), filename)


@dataclass
class AwsDynamoDbTable:
    client: 'botocore.client.DynamoDB'
    AccountId: str
    AccountName: str
    Region: str
    TableName: str

    def __post_init__(cls):
        cls.describe()

    def describe(cls):
        response = cls.client.describe_table(
            TableName=cls.TableName
        )
        info = response['Table']
        object.__setattr__(cls, 'info', info)
        tags = cls.client.list_tags_of_resource(ResourceArn=cls.info['TableArn'])['Tags']
        object.__setattr__(cls, 'tags', tags)

    @property
    def Ownername(cls):
        try:
            return next(item for item in cls.tags if item["Key"] == "owner")['Value']
        except Exception:
            return "missing owner"

    def output(cls) -> str:
        return {
            'TableName': cls.TableName,
            'AccountId': cls.AccountId,
            'AccountName': cls.AccountName,
            'Region': cls.Region,
            'Ownername': cls.Ownername,
        }

    def writeCsv(cls, filename):
        dictToCsv(cls.output(), filename)

